//{ **************************************************************************** }
//{ FileName............: EXRESOURCEGUARD.H                                      }
//{ Project.............: Generic                                                }
//{ Author(s)...........: D.Beck                                                 }
//{ Original contents...:                                                        }
//{ Version.............: x.xx                                                   }
//{ ---------------------------------------------------------------------------- }
//{ Declarations for Executive Resource class wrapper.                           }
//{                                                                              }
//{ Version  Date      Comment                                                   }
//{ x.xx     20070309  - Initial release                                         }
//{ **************************************************************************** }

#ifndef EXRESOURCEGUARD_H
  #define EXRESOURCEGUARD_H

#include "stddcls.h"
#include "driver.h"

class ExResourceGuardBase
{
public:
  inline ExResourceGuardBase( PERESOURCE resource ) : resource_( resource ) {};

  virtual ~ExResourceGuardBase();
  
private:

  // hide default c-tor
  ExResourceGuardBase();

protected:
  PERESOURCE resource_;
};

class ExResourceSharedGuard : public ExResourceGuardBase
{
public:
  inline ExResourceSharedGuard( PERESOURCE resource ) : ExResourceGuardBase( resource ) 
  {
    // disable APC delivery
    KeEnterCriticalRegion();
    
    // provides priority to any waiting writers requesting exclusive access
    ExAcquireSharedWaitForExclusive( resource_, TRUE );

    //KdPrint((DRIVERNAME " - ExResourceSharedGuard c-tor\n"));
  };
  virtual ~ExResourceSharedGuard() {};

private:

  // hide default c-tor
  ExResourceSharedGuard();
};

class ExResourceExcGuard : public ExResourceGuardBase
{
public:
  inline ExResourceExcGuard( PERESOURCE resource ) : ExResourceGuardBase( resource ) 
  {
    // disable APC delivery
    KeEnterCriticalRegion();
    
    ExAcquireResourceExclusiveLite( resource_, TRUE);

    //KdPrint((DRIVERNAME " - ExResourceExcGuard c-tor\n"));
  };
  virtual ~ExResourceExcGuard() {};
 
private:

  // hide default c-tor
  ExResourceExcGuard();
};

#endif EXRESOURCEGUARD_H