///////////////////////////////////////////////////////////////////////////////
// Module	 : App626.h
// Function  : Header file for Sensoray 626 applications.
// Usage	 : Included by all apps using 626 boards (platform independent).
// Author	 : Jim Lamberson, modified D.Beck
// Copyright : (C) 2002 Sensoray
///////////////////////////////////////////////////////////////////////////////

// Special types *********************************************************************************

typedef void	(* FPTR_ISR)();					// Pointer to application's interrupt callback function.
typedef void	(* FPTR_ERR)(unsigned long ErrFlags);	// Pointer to application's error callback function.

// Error codes returned by GetErrors() ***********************************************************

												// System errors that are not reset by S626_GetErrors():
#define ERR_OPEN			0x00000001			//	Can't open driver.
#define ERR_CARDREG			0x00000002			//	Can't attach driver to board.
#define ERR_ALLOC_MEMORY	0x00000004			//	Memory allocation error.
#define ERR_LOCK_BUFFER		0x00000008			//	Error locking DMA buffer.
#define ERR_THREAD			0x00000010			//	Error starting a thread.
#define ERR_INTERRUPT		0x00000020			//	Error enabling interrupt.
#define ERR_LOST_IRQ		0x00000040			//  Missed interrupt.
#define ERR_INIT			0x00000080			//  Board object not instantiated.
#define ERR_VERSION			0x00000100			//  Unsupported WinDriver version.
#define ERR_SUBIDS			0x00000200			//  PCI SubDevice/SubVendor ID mismatch.
#define ERR_CFGDUMP			0x00000400			//  PCI configuration dump failed.

												// Board errors that are reset by S626_GetErrors():
#define ERR_ILLEGAL_PARM	0x00010000			//  Illegal function parameter value was specified.
#define ERR_I2C				0x00020000			//	I2C error.
#define ERR_DACTIMEOUT		0x00100000			//	DAC FB_BUFFER write timeout.
#define ERR_COUNTERSETUP	0x00200000			//	Illegal setup specified for counter channel.

// ADC poll list constants *************************************************************************

#define ADC_EOPL			0x80				// End-Of-Poll-List marker.
#define ADC_RANGE_10V		0x00				// Range code for 10V range.
#define ADC_RANGE_5V		0x10				// Range code for 5V range.
#define ADC_CHANMASK		0x0F				// Channel number mask.

// Counter constants *******************************************************************************

												// LoadSrc values:
#define LOADSRC_INDX		0					//	Preload core in response to Index.
#define LOADSRC_OVER		1					//	Preload core in response to Overflow.
#define LOADSRCB_OVERA	2					//	Preload B core in response to A Overflow.
#define LOADSRC_NONE		3					//	Never preload core.

												// IntSrc values:
#define INTSRC_NONE 		0					//	Interrupts disabled.
#define INTSRC_OVER 		1					//	Interrupt on Overflow.
#define INTSRC_INDX 		2					//	Interrupt on Index.
#define INTSRC_BOTH 		3					//	Interrupt on Index or Overflow.

												// LatchSrc values:
#define LATCHSRC_AB_READ	0					//	Latch on read.
#define LATCHSRC_A_INDXA	1					//	Latch A on A Index.
#define LATCHSRC_B_INDXB	2					//	Latch B on B Index.
#define LATCHSRC_B_OVERA	3					//	Latch B on A Overflow.

												// IndxSrc values:
#define INDXSRC_HARD		0					//	Hardware or software index.
#define INDXSRC_SOFT		1					//	Software index only.

												// IndxPol values:
#define INDXPOL_POS 		0					//	Index input is active high.
#define INDXPOL_NEG 		1					//	Index input is active low.

												// ClkSrc values:
#define CLKSRC_COUNTER	0					//	Counter mode.
#define CLKSRC_TIMER		2					//	Timer mode.
#define CLKSRC_EXTENDER	3					//	Extender mode.

												// ClkPol values:
#define CLKPOL_POS			0					//	Counter/Extender clock is active high.
#define CLKPOL_NEG			1					//	Counter/Extender clock is active low.
#define CNTDIR_UP			  0					//	Timer counts up.
#define CNTDIR_DOWN 		1					//	Timer counts down.

												// ClkEnab values:
#define CLKENAB_ALWAYS	0					//	Clock always enabled.
#define CLKENAB_INDEX		1					//	Clock is enabled by index.

												// ClkMult values:
#define CLKMULT_4X 			0					//	4x clock multiplier.
#define CLKMULT_2X 			1					//	2x clock multiplier.
#define CLKMULT_1X 			2					//	1x clock multiplier.

												// Bit Field positions in COUNTER_SETUP structure:
#define BF_LOADSRC			9					//	Preload trigger.
#define BF_INDXSRC			7					//	Index source.
#define BF_INDXPOL			6					//	Index polarity.
#define BF_CLKSRC			  4					//	Clock source.
#define BF_CLKPOL			  3					//	Clock polarity/count direction.
#define BF_CLKMULT			1					//	Clock multiplier.
#define BF_CLKENAB			0					//	Clock enable.

												// Counter channel numbers:
#define CNTR_0A 			0					//	Counter 0A.
#define CNTR_1A 			1					//	Counter 1A.
#define CNTR_2A 			2					//	Counter 2A.
#define CNTR_0B 			3					//	Counter 0B.
#define CNTR_1B 			4					//	Counter 1B.
#define CNTR_2B 			5					//	Counter 2B.

// Counter overflow/index event flag masks for S626_CounterCapStatus() and S626_InterruptStatus().
#define INDXMASK(C)		( 1 << ( ( (C) > 2 ) ? ( (C) * 2 - 1 ) : ( (C) * 2 +  4 ) ) )
#define OVERMASK(C)		( 1 << ( ( (C) > 2 ) ? ( (C) * 2 + 5 ) : ( (C) * 2 + 10 ) ) )

// 2007-02-25: DBeck: added DIO masks
#define DIO_GRP_MSK(chan)  (WORD)(chan >> 4)         // Group number associated with chan.
#define DIO_CHAN_MSK(chan) (WORD)(1 << (chan & 15))  // Bit mask for chan within its group.

#define S626DIOINTNUM(x) x
#define S626CNTINDXINTNUM(C) 36 + ( ( (C) > 2 ) ? ( (C) * 2 - 1 ) : ( (C) * 2 +  4 ) )
#define S626CNTOVERINTNUM(C) 36 + ( ( (C) > 2 ) ? ( (C) * 2 + 5 ) : ( (C) * 2 + 10 ) )

// LOCAL BUS (GATE ARRAY) REGISTER ADDRESS OFFSETS /////////////////////////////////

												// Analog I/O registers:
#define LP_DACPOL				0x0082			//  Write DAC polarity.
#define LP_GSEL					0x0084			//  Write ADC gain.
#define LP_ISEL					0x0086			//  Write ADC channel select.
												// Digital I/O (write only):
#define LP_WRINTSELA			0x0042			//  Write A interrupt enable.
#define LP_WREDGSELA			0x0044			//  Write A edge selection.
#define LP_WRCAPSELA			0x0046			//  Write A capture enable.
#define LP_WRDOUTA				0x0048			//  Write A digital output.
#define LP_WRINTSELB			0x0052			//  Write B interrupt enable.
#define LP_WREDGSELB			0x0054			//  Write B edge selection.
#define LP_WRCAPSELB			0x0056			//  Write B capture enable.
#define LP_WRDOUTB				0x0058			//  Write B digital output.
#define LP_WRINTSELC			0x0062			//  Write C interrupt enable.
#define LP_WREDGSELC			0x0064			//  Write C edge selection.
#define LP_WRCAPSELC			0x0066			//  Write C capture enable.
#define LP_WRDOUTC				0x0068			//  Write C digital output.
												// Digital I/O (read only):
#define LP_RDDINA				0x0040			//  Read digital input.
#define LP_RDCAPFLGA			0x0048			//  Read edges captured.
#define LP_RDINTSELA			0x004A			//  Read interrupt enable register.
#define LP_RDEDGSELA			0x004C			//  Read edge selection register.
#define LP_RDCAPSELA			0x004E			//  Read capture enable register.
#define LP_RDDINB				0x0050			//  Read digital input.
#define LP_RDCAPFLGB			0x0058			//  Read edges captured.
#define LP_RDINTSELB			0x005A			//  Read interrupt enable register.
#define LP_RDEDGSELB			0x005C			//  Read edge selection register.
#define LP_RDCAPSELB			0x005E			//  Read capture enable register.
#define LP_RDDINC				0x0060			//  Read digital input.
#define LP_RDCAPFLGC			0x0068			//  Read edges captured.
#define LP_RDINTSELC			0x006A			//  Read interrupt enable register.
#define LP_RDEDGSELC			0x006C			//  Read edge selection register.
#define LP_RDCAPSELC			0x006E			//  Read capture enable register.
												// Counter Registers (read/write):
#define LP_CR0A					0x0000			//  0A setup register.
#define LP_CR0B					0x0002			//  0B setup register.
#define LP_CR1A					0x0004			//  1A setup register.
#define LP_CR1B					0x0006			//  1B setup register.
#define LP_CR2A					0x0008			//  2A setup register.
#define LP_CR2B					0x000A			//  2B setup register.
												// Counter PreLoad (write) and Latch (read) Registers:
#define	LP_CNTR0ALSW			0x000C			//  0A lsw.
#define	LP_CNTR0AMSW			0x000E			//  0A msw.
#define	LP_CNTR0BLSW			0x0010			//  0B lsw.
#define	LP_CNTR0BMSW			0x0012			//  0B msw.
#define	LP_CNTR1ALSW			0x0014			//  1A lsw.
#define	LP_CNTR1AMSW			0x0016			//  1A msw.
#define	LP_CNTR1BLSW			0x0018			//  1B lsw.
#define	LP_CNTR1BMSW			0x001A			//  1B msw.
#define	LP_CNTR2ALSW			0x001C			//  2A lsw.
#define	LP_CNTR2AMSW			0x001E			//  2A msw.
#define	LP_CNTR2BLSW			0x0020			//  2B lsw.
#define	LP_CNTR2BMSW			0x0022			//  2B msw.
												// Miscellaneous Registers (read/write):
#define LP_MISC1				0x0088			//  Read/write Misc1.
#define LP_WRMISC2				0x0090			//  Write Misc2.
#define LP_RDMISC2				0x0082			//  Read Misc2.
