//$Id$

/* ****************************************************************************
 *
 *  Cls626mm.cpp
 *  Created by David Beck, on Fri Feb 09 2007, 19:05:18 Pacific Standard Time
 *
 *  Copyright (c) 2006, indigocode - www.indigocode.com
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *
 *  File Contents: Implementation of the Cls626mm component.
 *  Please see Cls626mm.h for full documentation of this system.
 *
 *  %version: 1 %
 *  %date_modified: Fri Feb 09 2007, 19:05:18 Pacific Standard Time %
 *  %created_by: David Beck %
 *  Version: %I%
 *  Date modified: %G%
 *
 *************************************************************************** */

#include "stdafx.h"

#include <ace/OS.h>
#include "Trace.h"

#undef ACE_NTRACE
#define ACE_NTRACE 0

#include <string>
#include <vector>

#include <boost/scoped_array.hpp>
#include <boost/format.hpp>

#include "Cls626mm.h"
#include "S626IntrFifoBufSet.h"

extern void rptBoardMutexFailure( int lineNum );

Cls626mm::Cls626mm( HANDLE hnd, const CDeviceListEntry& devInfo, FPTR_ISR IntFunc,
                   unsigned long Priority) 
    : drvHnd_(hnd), devInfo_(devInfo), intrHdlrCallBack_(IntFunc), 
    shutdownIntrHdlrThrd_(true)
{
  // initialize FIFO buffer map
  s626IntrFifoBufSets_.assign( NULL );

  // open driver
  // TODO: figure out what board revision this software can work with
  BoardOpen( 0 ); // parameter: is board rev. A?

  threadHandles_[0] = INVALID_HANDLE_VALUE;

  // start up interrupt status delivery thread
  if( NULL != IntFunc )
  {
    shutdownIntrHdlrThrd_ = false;
    if( 0 > activate( THR_NEW_LWP | THR_JOINABLE |THR_INHERIT_SCHED, 1, 0,
                        Priority, -1, NULL, threadHandles_) )
    {
      Cls626mm::logMsg( ACE_TEXT("Cls626mm::Cls626mm: activate failed") );
    }
  }
}

Cls626mm::~Cls626mm()
{
  StopIntrNotifyThrd();

  // release any remaining FIFO buffers  
  for( unsigned long i = 0; i < s626IntrFifoBufSets_.size(); ++i )
    CloseIrqFifoBuffers( i );

  BoardClose();

  if ( INVALID_HANDLE_VALUE != drvHnd_ )
  {
    ::CloseHandle( drvHnd_ );
    drvHnd_ = INVALID_HANDLE_VALUE;
  }
}

// Functions that must be implemented in the derived OS interface class:

// Overall critical section begin.
void Cls626mm::CriticalBegin() 
{  
  if( 0 != genLock_.acquire() ) rptBoardMutexFailure( __LINE__ );
}

// Overall critical section end.
void Cls626mm::CriticalEnd()
{
  genLock_.release();
}

// ADC critical section begin.
void Cls626mm::CriticalBeginAdc()
{
  if( 0 != adcLock_.acquire() ) rptBoardMutexFailure( __LINE__ );
}

// ADC critical section end.
void Cls626mm::CriticalEndAdc()
{
  adcLock_.release();
}

unsigned long	Cls626mm::ReadMem( unsigned long address )
{ 
  unsigned long data = 0ul;
  DWORD cntRead;
  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_SAA7146_READ,
    (LPVOID) &address, sizeof(unsigned long), (LPVOID) &data, sizeof(data), 
    &cntRead, NULL ) )
  {        
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::ReadMem: address=0x%8X") ) % address ).c_str(), 
      ::GetLastError() );
  }
  return data;
}

void Cls626mm::WriteMem( unsigned long address, unsigned long data )
{
  ACE_ASSERT( ((data & 0x0000FFFF) != 0xCDCD) );

  unsigned long buffer[2];
  buffer[0] = address;
  buffer[1] = data;
  DWORD cntWritten;
  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_SAA7146_WRITE,
    (LPVOID) buffer, sizeof(buffer), NULL, 0, &cntWritten, NULL ) )
  {    
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::WriteMem: address=0x%8X") ) % address ).c_str(), 
      ::GetLastError() );
  }
}

// Functions that must be implemented in the derived OS interface class:
// Allocate DMA buffer.
void	Cls626mm::AllocDMAB( unsigned long nBytes, DMABUF *pDmaBuf )
{

	// Allocate DMA buffers using AllocDMAB(), a pure virtual function provided by the OS Interface class.
	// Each call to AllocDMAB() allocates physically contiguous buffer space at least as large as the
	// requested size.  The buffer must begin on a page boundary, and pages must contain at least 4KB
	// of contiguous memory.  This class (and the AllocDMAB() function) must be modified if the target
	// OS can not provide the minimum 4KB physical page size.  AllocDMAB() sets the "allocation error"
	// flag in MyErrFlags if any buffer allocation fails.

  DWORD cntRead = 0;
  DMABUFFER buf = {0};
  
  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_DMA_ALLOCATE, 
    (LPVOID) &nBytes, sizeof(nBytes), (LPVOID) &buf, sizeof(Cls626mm::DMABUFFER), 
    &cntRead, NULL ) )
  {
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::AllocDMAB: size=%d") ) % nBytes ).c_str(), ::GetLastError() );
    // TODO: set MyErrFlags
  } 
  else
  {
    // transfer results to caller's structure
    pDmaBuf->DMAHandle = buf.BufferId;
    pDmaBuf->LogicalBase = buf.VaBuffer;
    pDmaBuf->PhysicalBase = buf.PaBuffer.LowPart;
    pDmaBuf->bufByteSize = buf.BufferSize;
  }
}

// Deallocate DMA buffer.
void	Cls626mm::CloseDMAB( DMABUF *pDmaBuf )
{
  DWORD cntRead = 0; 
  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_DMA_RELEASE, 
    (LPVOID) &(pDmaBuf->DMAHandle), sizeof(unsigned long), NULL, 0, &cntRead, NULL ) )
  {
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::CloseDMAB: ID=0x%X") ) % pDmaBuf->DMAHandle ).c_str(),
      ::GetLastError() );
  }
}

void Cls626mm::ReadDmaMem( const DMABUF& dmaBuf, PUCHAR trgBuf, 
                          ULONG cntXferBytes, ULONG trgStartIdx /*= 0*/, 
                          ULONG dmaStartIdx /*= 0*/)
{
  DWORD cntRead = 0;
  TRANSFERBUFFER readBuf = {0};

  readBuf.bufferId = dmaBuf.DMAHandle;
  readBuf.bufferSourceIndex = dmaStartIdx;
  readBuf.bufferTargetIndex = trgStartIdx;
  readBuf.bufferTransferLength = cntXferBytes;
  readBuf.bufferTransfer = trgBuf;

  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_DMA_READ, 
    (LPVOID) &(readBuf), sizeof(TRANSFERBUFFER), NULL, 0, &cntRead, NULL ) )
  {
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::ReadDmaMem: ID=0x%X") ) % dmaBuf.DMAHandle ).c_str(),
      ::GetLastError() );
  }
}
	
void Cls626mm::WriteDmaMem( const DMABUF& dmaBuf, PUCHAR srcBuf, 
                           ULONG cntXferDwords, ULONG srcStartIdx /*= 0*/, 
                           ULONG dmaStartIdx /*= 0*/)
{
  DWORD cntWritten = 0;
  TRANSFERBUFFER writeBuf = {0};

  writeBuf.bufferId = dmaBuf.DMAHandle;
  writeBuf.bufferSourceIndex = srcStartIdx;
  writeBuf.bufferTargetIndex = dmaStartIdx;
  writeBuf.bufferTransferLength = cntXferDwords;
  writeBuf.bufferTransfer = srcBuf;

  if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_DMA_WRITE, 
    (LPVOID) &(writeBuf), sizeof(TRANSFERBUFFER), NULL, 0, &cntWritten, NULL ) )
  {
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::WriteDmaMem: ID=0x%X") ) % dmaBuf.DMAHandle ).c_str(),
      ::GetLastError() );
  }
}

long Cls626mm::OpenIrqFifoBuffers( unsigned long irqId, unsigned long* regAddrs, 
                              long numAddrs, long numBufsReq )
{
  if( 0 > irqId || irqId >= s626IntrFifoBufSets_.size() )
    return -1;

  S626IntrFifoBufSet* fifoBufSet = s626IntrFifoBufSets_[ irqId ];
  if( NULL == fifoBufSet )
  {
    // construct register address list
    S626IntrFifoBufSet::RegAddrList regAddrList( regAddrs, regAddrs + numAddrs );
    //std::copy( regAddrs, regAddrs + numAddrs, std::back_inserter(regAddrList) );

    ACE_NEW_RETURN( fifoBufSet, S626IntrFifoBufSet( this, irqId, regAddrList,\
      numBufsReq ), -1 );

    // add open buffer set to buffer set map
    s626IntrFifoBufSets_[ irqId ] = fifoBufSet;  
  }  

  // report to caller actual number of FIFO's opened
  return fifoBufSet->getFifoBufCnt();
}

void Cls626mm::CloseIrqFifoBuffers( unsigned long irqId )
{
  if( 0 > irqId || irqId >= s626IntrFifoBufSets_.size() )
    return;

  S626IntrFifoBufSet* fifoBufSet = s626IntrFifoBufSets_[ irqId ];
  if( NULL != fifoBufSet )
  {
    delete fifoBufSet;

    // remove buffer set from buffer set map
    s626IntrFifoBufSets_[ irqId ] = NULL;  
  }  
}

unsigned long Cls626mm::FetchIrqFifoBuffers( unsigned long irqId, 
                    unsigned char* buffer, unsigned long bufferByteCnt )
{
  if( 0 > irqId || irqId >= s626IntrFifoBufSets_.size() )
    return 0;
  S626IntrFifoBufSet* fifoBufSet = s626IntrFifoBufSets_[ irqId ];
  return fifoBufSet->fetchBuffers( buffer, bufferByteCnt );
}
  
long Cls626mm::AllocFifoBuffers( long bufferCnt, void* dmaSrcAddr, 
                                  unsigned long fifoBufByteSize )
{  
  DWORD cntRead = 0;
  FIFOTRANSFERBUFFER buf = {0};
  buf.bufferId = -1;
  buf.buffers = bufferCnt;
  buf.bufferTransfer = dmaSrcAddr;
  buf.bufferTransferLength = fifoBufByteSize;
  
  if( !::DeviceIoControl( drvHnd_, IOCTL_FIFO_ALLOCATE, 
    (LPVOID) &buf, sizeof(buf), (LPVOID) &buf, sizeof(buf), 
    &cntRead, NULL ) )
  {
    LARGE_INTEGER addr;
    addr.QuadPart = (LONGLONG) dmaSrcAddr;

    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::AllocFifoBuffers: dmaSrcAddr=0x%08X") ) % addr.LowPart ).c_str(),
      ::GetLastError() );

    // TODO: set MyErrFlags
  } 
  else
  {
    // returns startFifoBufId of range allocated
    return buf.bufferId;
  }

  // failure
  return -1;
}


long Cls626mm::CloseFifoBuffers( long startFifoBufId, long bufCnt, long& bufCntClosed )
{
  DWORD cntRead = 0;
  FIFOTRANSFERBUFFER buf = {0};
  buf.bufferId = startFifoBufId;
  buf.buffers = bufCnt;
  
  if( !::DeviceIoControl( drvHnd_, IOCTL_FIFO_RELEASE, 
    (LPVOID) &buf, sizeof(buf), (LPVOID) &buf, sizeof(buf), 
    &cntRead, NULL ) )
  {
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("Cls626mm::CloseFifoBuffers: fifoBufId=%d") ) % startFifoBufId ).c_str(),
      ::GetLastError() );

    // TODO: set MyErrFlags
  } 
  else
  {
    bufCntClosed = buf.buffers;

    // return starting FIFO Id of range closed
    return buf.bufferId;
  }
  
  // failure
  return -1;
}

void Cls626mm::WriteS626IrqConfig( long irqId, IRQBUFFER& irqInfo )
{
  DWORD cntRead = 0;
  IRQTRANSFERBUFFER buf = { 0 };

  // replace S626 relative Irq ID with driver's absolute address
  buf.IrqId = irqId + S626IRQNUMOFFSET;
  buf.IrqInformation = irqInfo;

  if( 0 == ::DeviceIoControl( drvHnd_, IOCTL_IRQ_WRITE, &buf, 
      sizeof(buf), NULL, 0, &cntRead, NULL ) )
  {
    Cls626mm::logMsg( ACE_TEXT("Cls626mm::WriteS626IrqConfig DeviceIoControl-IRQ_WRITE"), 
      ::GetLastError() );
  }
}

void Cls626mm::ReadS626IrqConfig( long irqId, IRQBUFFER& irqInfo )
{
  DWORD cntRead = 0;
  IRQTRANSFERBUFFER buf = { 0 };

  // replace S626 relative Irq ID with driver's absolute address
  buf.IrqId = irqId + S626IRQNUMOFFSET;  

  if( 0 == ::DeviceIoControl( drvHnd_, IOCTL_IRQ_READ, &buf, 
      sizeof(buf), &buf, sizeof(buf), &cntRead, NULL ) )
  {
    Cls626mm::logMsg( ACE_TEXT("Cls626mm::ReadS626IrqConfig DeviceIoControl-IRQ_READ"), 
      ::GetLastError() );
  }

  // copy results back to caller's buffer
  ::memcpy( &irqInfo, &buf.IrqInformation, sizeof(irqInfo) );

  // restore S626 relative Irq ID
  buf.IrqId = irqId;
}

unsigned long Cls626mm::GetAddress()
{
  unsigned long busLocation = Cls626::GetAddress();

  if( 0 == busLocation )
  {
    DWORD cntRead = 0ul;
    if( !::DeviceIoControl( drvHnd_, IOCTL_CLS626_BUS_LOC,
      NULL, 0, (LPVOID) &busLocation, sizeof(busLocation), &cntRead, NULL ) )
    {
      Cls626mm::logMsg( ACE_TEXT("Cls626mm::GetAddress"), ::GetLastError() );     
    }
    else
    {
      MyAddress = busLocation;
    }
  }
  return busLocation;
}

void Cls626mm::InterruptStatus( unsigned short* intFlags )
{
  ::memcpy( intFlags, irqStatus_.intrStatusWord, sizeof(irqStatus_.intrStatusWord) );  
}

void Cls626mm::StopIntrNotifyThrd()
{
  if( INVALID_HANDLE_VALUE == threadHandles_[0] ) return;

  shutdownIntrHdlrThrd_ = true;
  
  // cancel notification wait in driver
  DWORD cntRead = 0;
  DWORD junk = 0;
  if( 0 == ::DeviceIoControl( drvHnd_, IOCTL_CLS626_GENERATE_EVENT, 
    (LPVOID) &junk, sizeof(junk), NULL, 0, &cntRead, NULL ) )
  {
    Cls626mm::logMsg( 
      ACE_TEXT("Cls626mm::StopIntrNotifyThrd, initial cancellation"), ::GetLastError() );
  } 
  
  while( WAIT_TIMEOUT == ::WaitForSingleObject( threadHandles_[0], 50 ) )
  {
    if( 0 == ::DeviceIoControl( drvHnd_, IOCTL_CLS626_GENERATE_EVENT, 
        (LPVOID) &junk, sizeof(junk), NULL, 0, &cntRead, NULL ) )
      {
        Cls626mm::logMsg( 
          ACE_TEXT("Cls626mm::StopIntrNotifyThrd, cancellation loop"), ::GetLastError() );
      } 
  }

  ::CloseHandle( threadHandles_[0] );
  threadHandles_[0] = INVALID_HANDLE_VALUE;

  // let ACE_Task_Base do it's clean-up 
  //  (it'll return an error though because the thread has already been shutdown)
  wait();  
}

int Cls626mm::svc()
{
  if( shutdownIntrHdlrThrd_ ) return 0;

  // open driver
  HANDLE hnd = ::CreateFile( devInfo_.m_linkname.c_str(), GENERIC_WRITE | GENERIC_READ, 
    0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0 );

  if( INVALID_HANDLE_VALUE == hnd )
  {   
    Cls626mm::logMsg( ACE_TEXT("Cls626mm::svc CreateFile"), ::GetLastError() );    
    return -1;
  }
  
  // Note: increase array size to handle multiple IRQ buffers, presently only
  //  working with the master interrupt to ensure event notification for any
  //  S626 interrupt.
  IRQTRANSFERBUFFER irqInfo[1]; // 
  ::memset( irqInfo, 0, sizeof(irqInfo) );

  // retrieve all the buffers of interest first and only modify the event related
  //  information in case another thread is also doing IRQ related monitoring
  //  (specifically FIFO or register writing)
  for( int i = 0; i < sizeof(irqInfo)/sizeof(IRQTRANSFERBUFFER); ++i )
  {
    DWORD cntRead = 0;
    DWORD junk = 0;
   
    irqInfo[ i ].IrqId = (0 == i) ? SAA7146A_DEBI_BUS_INTR_BIT :
                                                      ((i - 1) + SAA7146A_S626_IRQNUMOFFSET);
   
    if( 0 == ::DeviceIoControl( hnd, IOCTL_IRQ_READ, &(irqInfo[i]), 
      sizeof(IRQTRANSFERBUFFER), &(irqInfo[i]), sizeof(IRQTRANSFERBUFFER), &cntRead, NULL ) )
    {
      Cls626mm::logMsg( ACE_TEXT("Cls626mm::svc DeviceIoControl-IRQ_READ"), ::GetLastError() );
    }

    IRQBUFFER* buf = &(irqInfo[ i ].IrqInformation);
    buf->active = TRUE;
    buf->eventActive = TRUE;
    buf->irqAutoDisable = FALSE;

    if( 0 == ::DeviceIoControl( hnd, IOCTL_IRQ_WRITE, &(irqInfo[i]), 
      sizeof(IRQTRANSFERBUFFER), NULL, 0, &cntRead, NULL ) )
    {
      Cls626mm::logMsg( ACE_TEXT("Cls626mm::svc DeviceIoControl-IRQ_WRITE"), ::GetLastError() );
    }
  }

  while( !shutdownIntrHdlrThrd_ )
  {
    DWORD cntRead = 0;
    if( 0 == ::DeviceIoControl( hnd, IOCTL_CLS626_WAIT_NOTIFY, NULL, 0, 
      (LPVOID) &irqStatus_, sizeof(irqStatus_), &cntRead, NULL ) )
    {
      Cls626mm::logMsg( ACE_TEXT("Cls626mm::svc DeviceIoControl-WAIT_NOTIFY"), ::GetLastError() );
      ::CloseHandle( hnd );
      return -1;
    }

    if( !shutdownIntrHdlrThrd_ && intrHdlrCallBack_ )
    {
      intrHdlrCallBack_();
      //traceAllSAA7146AIntrs( hnd, irqInfo );
    }
  }

  ::CloseHandle( hnd );
  return 0;
}

// static helpers
void Cls626mm::logMsg( const ACE_TCHAR* msg, unsigned long errCode )
{
    ACE_DEBUG( 
      (MY_DEBUG ACE_TEXT("%s, ErrCode=0x%X, %P\n"), msg, errCode, ACE_TEXT(" ")) );
}

void Cls626mm::traceAllSAA7146AIntrs( HANDLE hnd, IRQTRANSFERBUFFER xferBufs[] )
{
  for( int i = 0; i < sizeof(xferBufs)/sizeof(IRQTRANSFERBUFFER); ++i )
  {
    DWORD cntRead = 0;
    DWORD junk = 0;
    xferBufs[ i ].IrqId = i;
    IRQBUFFER* buf = &(xferBufs[ i ].IrqInformation);
    buf->active = TRUE;
    buf->eventActive = TRUE;

    if( 0 == ::DeviceIoControl( hnd, IOCTL_IRQ_READ, &(xferBufs[i]), 
      sizeof(IRQTRANSFERBUFFER), &(xferBufs[i]), sizeof(IRQTRANSFERBUFFER), &cntRead, NULL ) )
    {
      Cls626mm::logMsg( 
        ACE_TEXT("traceAllSAA7146AIntrs DeviceIoControl-IRQ_WRITE"), ::GetLastError() );
    }
    else
    {
      if( 0 < buf->irqsReceived || 0 < buf->irqsActiveReceived )
      {
        ACE_DEBUG( (MY_DEBUG ACE_TEXT("IRQ:%d, numRecv=%d, numActRcv=%d\n"), xferBufs[i].IrqId,\
          buf->irqsReceived, buf->irqsActiveReceived) );          
      }
    }
  }
}