//$Id$

/* ****************************************************************************
 *
 *  S626IntrFifoBufSet.h
 *  Created by David Beck, on Sun Mar 18 2007, 16:25:59 Pacific Daylight Time
 *
 *  Copyright (c) 2006, indigocode - www.indigocode.com
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *
 *  File Contents: Interface and documentation of the S626IntrFifoBufSet component.
 *
 *  %version: 1 %
 *  %date_modified: Sun Mar 18 2007, 16:25:59 Pacific Daylight Time %
 *  %created_by: David Beck %
 *  Version: %I%
 *  Date modified: %G%
 *
 *************************************************************************** */
 
#ifndef IC__S626INTRFIFOBUFSET_H
#define IC__S626INTRFIFOBUFSET_H

#include <vector>
#include <boost/utility.hpp>

// forward declarations
class Cls626mm;

// Specification of S626 registers to buffer in a FIFO upon S626 interrupt event;
//  stored in DMA and DMA location provided as "src" to FIFO
//  (specified using FIFO and IRQ_WRITE mechanism), 
//  format in FIFO is LONGLONG timestamp, UINT regData[]
typedef struct _S626IRQREGLIST
{
  LONG          regCount;                                // Count of registers in list (must be >= 1)
  ULONG         junkFiller;                              // Reserving room for high part of 
  UINT32        s626RegAddr[1];                          // List of 16 bit (sized for 4 byte alignment) register addresses follows.
} S626IRQREGLIST, *PS626IRQREGLIST;
 
/** Purpose: Describe the purpose of the class.
 *  (Everything that you write here will show up in the
 *  doc++ generated documentation.)
 */
class S626IntrFifoBufSet : public boost::noncopyable
{
public:

  typedef std::vector< unsigned long > RegAddrList;

  static const long DEFAULT_FIFO_BUF_CNT = 20;

  /// Constructor.
  S626IntrFifoBufSet( Cls626mm* board, const unsigned long s626IntrId, 
    const RegAddrList& regAddrList, const long fifoBufCnt = DEFAULT_FIFO_BUF_CNT );

  virtual ~S626IntrFifoBufSet();

  inline long getFifoBufCnt() const { return fifoBufCnt_; }
  inline long getDmaByteCnt() const { return dmaBufByteCnt_; }

  unsigned long fetchBuffers( unsigned char* buffer, unsigned long bufferByteCnt );

private:

  S626IntrFifoBufSet();	

private:
	
  Cls626mm*           board_;
  const unsigned long s626IntrId_;
  long                startingFifoId_;
  DMABUF              dmaBuf_;
  long                fifoBufCnt_;
  unsigned long       dmaBufByteCnt_;
};
 
#include "S626IntrFifoBufSet.inl"
#endif // #ifndef IC__S626INTRFIFOBUFSET_H
 
