// s626mm.cpp : 
//

#include "stdafx.h"
#include <vector>
#include <functional>
#include <algorithm>
#include <string>
#include <boost/format.hpp>

#include <ace/OS.h>
#include <ace/Refcounted_Auto_Ptr.h>
#include "Trace.h"

#undef ACE_NTRACE
#define ACE_NTRACE 0

// DLL export declarations:
#define XFUNC626(RTNTYPE,FUNCNAME) extern "C" __declspec( dllexport ) RTNTYPE FUNCNAME
#include "Win626.h"
#include "Cls626mm.h"
#include "DeviceList.h"

static const int MAX_NUM_BOARDS = 16;
static std::vector< Cls626mm_Ptr > boards( MAX_NUM_BOARDS );
typedef ACE_Recursive_Thread_Mutex BOARD_MUTEX;
static BOARD_MUTEX boardMutex;

// {4C49534D-4152-0000-0000-000000000000}
// {4D36E96C-E325-11CE-BFC1-08002BE10318}
static const GUID LISMAR_GUID = 
//{ 0x4c49534d, 0x4152, 0x0000, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } };
{ 0x4d36e96c, 0xe325, 0x11ce, { 0xbf, 0xc1, 0x08, 0x00, 0x2b, 0xe1, 0x03, 0x18 } };

// forward declarations
void rptBoardMutexFailure( int lineNum );

///////////////////////////////////////////////////////////////////////////////
//
// Defines the entry point for the DLL application.
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, 
                      LPVOID lpReserved )
{
  // TODO: close any open boards if unloading?
  return TRUE;
}

// Status and control functions.
XFUNC626( unsigned long, S626_GetAddress )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->GetAddress();
}

XFUNC626( unsigned long, 	S626_GetErrors )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return ERR_ILLEGAL_PARM;
  }
  return board->GetErrors();  
}

XFUNC626( void, S626_OpenBoard )( unsigned long hbd, unsigned long /*PhysLoc*/, 
                    FPTR_ISR IntFunc, unsigned long Priority )
{
  if( ( 0 > hbd ) || ( MAX_NUM_BOARDS <= hbd ) ) 
  {
    // can't report the error for a non-existent board, but a subsequent call
    //  to GetErrors will help indicate the problem.
    return;
  }

  // protect from here on...
  ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
    rptBoardMutexFailure(__LINE__); return );

  // look for existing board
  Cls626mm_Ptr board = boards[hbd];
  if( !board.null() ) return;

  // enumerate installed boards and gather DevicePath's if not previously done
  if( DEVICE_LIST::instance()->m_list.empty() ) 
    DEVICE_LIST::instance()->Initialize( LISMAR_GUID );
  
  if( DEVICE_LIST::instance()->m_list.size() > hbd )
  {
    // open driver
    HANDLE hnd = ::CreateFile( DEVICE_LIST::instance()->m_list[hbd].m_linkname.c_str(), 
      GENERIC_WRITE | GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0 );
    if( INVALID_HANDLE_VALUE == hnd )
    {   
      Cls626mm::logMsg( ACE_TEXT("S626_OpenBoard: CreateFile"), ::GetLastError() );
      return;
    }
    
    // create new board class and enter in boards list
    Cls626mm* newBoard;
    ACE_NEW_NORETURN( newBoard, Cls626mm( hnd, DEVICE_LIST::instance()->m_list[hbd], 
      IntFunc, Priority ) );
    if( NULL != newBoard )
    {
      boards[hbd] = Cls626mm_Ptr( newBoard );
    }
    else
    {
      Cls626mm::logMsg( ACE_TEXT("S626_OpenBoard: Cls626mm alloc failure"));
    }
  } 
  else
  {
    // report board handle doesn't match an installed board  
    Cls626mm::logMsg( boost::str( boost::wformat( 
      ACE_TEXT("S626_OpenBoard: Invalid Board Handle=0x%0X, Max Hnd=0x%0X") ) 
      % hbd % ((unsigned long)(DEVICE_LIST::instance()->m_list.size()) - 1) ).c_str(),
      ::GetLastError() );
  }
}

XFUNC626( void, S626_CloseBoard )( unsigned long hbd )
{
  // protect this along with OpenBoard  
  ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
    rptBoardMutexFailure(__LINE__); return );

  if( boards[hbd].null() ) return;

  // remove board class from list
  Cls626mm_Ptr board = boards[hbd];
  boards[hbd] = Cls626mm_Ptr();

  // destroy singleton device list (DevicePath's) if this is the last board
  //  to close
  if( boards.end() == std::find_if( boards.begin(), boards.end(), 
    std::bind1st( std::not_equal_to< Cls626mm_Ptr >(), Cls626mm_Ptr() ) ) )
      DEVICE_LIST::close();
}

XFUNC626( void, S626_InterruptEnable )( unsigned long hbd, unsigned short enable )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->InterruptEnable( enable );
}

XFUNC626( void, S626_InterruptStatus )( unsigned long hbd, unsigned short *status )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->InterruptStatus( status );
}

XFUNC626( void, S626_SetErrCallback )( unsigned long hbd, FPTR_ERR Callback )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->SetErrCallback( Callback );
}

// Diagnostics.
XFUNC626( unsigned char, S626_I2CRead )( unsigned long hbd, unsigned char addr )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->I2Cread( addr );
}

XFUNC626( void, S626_I2CWrite )( unsigned long hbd, unsigned char addr, 
                   unsigned char value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  return board->I2Cwrite( addr, value );
}

XFUNC626( unsigned short, S626_RegRead )( unsigned long hbd, unsigned short addr )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0);
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return (unsigned short) board->ReadMem( addr );
}

XFUNC626( void, S626_RegWrite )( unsigned long hbd, unsigned short addr, 
                   unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return);
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WriteMem( addr, value );
  return;
}

// Analog I/O functions.
XFUNC626( void, S626_ReadADC )( unsigned long hbd, short *databuf )
{
   Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->ReadADC( reinterpret_cast< unsigned short* >(databuf) );
}

XFUNC626( void, S626_ResetADC )( unsigned long hbd, unsigned char *pollist )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->ResetADC( pollist );
}

XFUNC626( void, S626_StartADC )( unsigned long hbd)
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->StartADC();
}

XFUNC626( void, S626_WaitDoneADC )( unsigned long hbd, short *pdata)
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WaitDoneADC( reinterpret_cast< unsigned short* >(pdata) );
}

XFUNC626( void, S626_WriteDAC )( unsigned long hbd, unsigned short chan, 
                                long value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WriteDAC( chan, static_cast< short >(value) );
}

XFUNC626( void, S626_WriteTrimDAC )( unsigned long hbd, unsigned char chan, 
                       unsigned char value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WriteTrimDAC( chan, value );
}

// Digital I/O functions.
XFUNC626( unsigned short, S626_DIOReadBank )( unsigned long hbd, 
                                             unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupRead( group );
}

XFUNC626( unsigned short, S626_DIOWriteBankGet )( unsigned long hbd, 
                                                 unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupWrite( group );
}

XFUNC626( void, S626_DIOWriteBankSet )( unsigned long hbd, unsigned short group, 
                          unsigned short value )
{
    Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->DIOGroupWrite( group, value );
}

XFUNC626( unsigned short, S626_DIOEdgeGet )( unsigned long hbd, 
                                            unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupEdge( group );
}

XFUNC626( void, S626_DIOEdgeSet )( unsigned long hbd, unsigned short group, 
                     unsigned short value )
{
    Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  return board->DIOGroupEdge( group, value );
}

XFUNC626( void, S626_DIOCapEnableSet )( unsigned long hbd, unsigned short group, 
                          unsigned short chanmask, unsigned short enable )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->DIOGroupCapEnable( group, chanmask, enable );
}

XFUNC626( unsigned short, S626_DIOCapEnableGet )( unsigned long hbd, 
                                                 unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupCapEnable( group );
}

XFUNC626( unsigned short, S626_DIOCapStatus )( unsigned long hbd, 
                                              unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupCapStatus( group );
}


XFUNC626( void, S626_DIOCapReset )( unsigned long hbd, unsigned short group, 
                                   unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->DIOGroupCapReset( group, value );
}

XFUNC626( unsigned short,	S626_DIOIntEnableGet )( unsigned long hbd, 
                                                 unsigned short group )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOGroupIntEnable( group );
}

XFUNC626( void, S626_DIOIntEnableSet )( unsigned long hbd, unsigned short group, 
                                       unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->DIOGroupIntEnable( group, value );

  // update S626IrqBufStatus for this group of interrupts
  if( !(ERR_ILLEGAL_PARM & board->GetErrors()) )
  {
    for( long irqId = (group * 16); irqId < ((group + 1)*16); ++irqId )
    {
      AceGdRecThrMtx guard( board->CriticalS626IrqConfig() );

      // config IRQ
      Cls626mm::IRQBUFFER irqConfig;
      ::memset( &irqConfig, 0, sizeof(irqConfig) );

      board->ReadS626IrqConfig( irqId, irqConfig );
      irqConfig.eventActive = ( (value >> (irqId % 16)) & 1 );
      irqConfig.active = 
        irqConfig.eventActive || irqConfig.fifoActive || irqConfig.signalActive;

      board->WriteS626IrqConfig( irqId, irqConfig );    
    }
  }
  else
  {
    Cls626mm::logMsg( 
      boost::str( boost::wformat( 
      ACE_TEXT("S626_DIOIntEnableSet: group=%d, value=0x%04X") )
      % group % value ).c_str(), ::GetLastError() );
  }
}

XFUNC626( unsigned short,	S626_DIOModeGet	)( unsigned long hbd, unsigned short chan )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->DIOChanMode( chan );
}

XFUNC626( void,	S626_DIOModeSet	)( unsigned long hbd, unsigned short chan, 
                                  unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->DIOChanMode( chan, value );
}

// Counter functions.
XFUNC626( void,	S626_CounterModeSet	)( unsigned long hbd, unsigned short chan, 
                                      unsigned short mode )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterModeSet( chan, mode );
}

XFUNC626( unsigned short,	S626_CounterModeGet	)( unsigned long hbd, 
                                                unsigned short chan )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->CounterModeGet( chan );
}

XFUNC626( void,	S626_CounterEnableSet	)( unsigned long hbd, unsigned short chan, 
                                        unsigned short enable )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterEnableSet( chan, enable );
}

XFUNC626( void, S626_CounterPreload	)( unsigned long hbd, unsigned short chan, 
                                      unsigned long value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterPreload( chan, value );
}

XFUNC626( void,	S626_CounterLoadTrigSet	)( unsigned long hbd, unsigned short chan, 
                                          unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterLoadTrigSet( chan, value );
}

XFUNC626( void,	S626_CounterLatchSourceSet )( unsigned long hbd, unsigned short chan, 
                                            unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterLatchSrcSet( chan, value );
}

XFUNC626( unsigned long, S626_CounterReadLatch )( unsigned long hbd, 
                                                 unsigned short chan )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->CounterReadLatch( chan );
}

XFUNC626( unsigned short,	S626_CounterCapStatus	)( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->CounterCapStatus();
}

XFUNC626( void,	S626_CounterCapFlagsReset	)( unsigned long hbd, 
                                            unsigned short chan )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterResetCapFlags( chan );
}

XFUNC626( void,	S626_CounterSoftIndex	)( unsigned long hbd, unsigned short chan )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterSoftIndex( chan );
}

XFUNC626( void,	S626_CounterIntSourceSet )( unsigned long hbd, unsigned short chan, 
                                           unsigned short value )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CounterIntSourceSet( chan, value );

  // update S626IrqBufStatus for this counter interrupt
  if( !(ERR_ILLEGAL_PARM & board->GetErrors()) )
  {
    AceGdRecThrMtx guard( board->CriticalS626IrqConfig() );

    // config IRQ,  (IRQ 40-45=INDX, 46-51=OVER)
    Cls626mm::IRQBUFFER irqConfig;

    // update overflow interrupt
    ::memset( &irqConfig, 0, sizeof(irqConfig) );
    board->ReadS626IrqConfig( S626CNTOVERINTNUM(chan), irqConfig );
    
    // INTSRC_OVER = 1, INTSRC_INDEX = 2
    irqConfig.eventActive = (value & 1);
    irqConfig.active = 
      irqConfig.eventActive || irqConfig.fifoActive || irqConfig.signalActive;

    board->WriteS626IrqConfig( S626CNTOVERINTNUM(chan), irqConfig );

    // update index interrupt
    ::memset( &irqConfig, 0, sizeof(irqConfig) );
    board->ReadS626IrqConfig( S626CNTINDXINTNUM(chan), irqConfig );
    
    // INTSRC_OVER = 1, INTSRC_INDEX = 2
    irqConfig.eventActive = (value & 2);
    irqConfig.active = 
      irqConfig.eventActive || irqConfig.fifoActive || irqConfig.signalActive;

    board->WriteS626IrqConfig( S626CNTINDXINTNUM(chan), irqConfig );
  }
  else
  {
    Cls626mm::logMsg( 
      boost::str( boost::wformat( 
      ACE_TEXT("S626_CounterIntSourceSet: chan=%d, value=0x%04X") )
      % chan % value ).c_str(), ::GetLastError() );
  }
}

// Battery functions:
XFUNC626( unsigned short,	S626_BackupEnableGet )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->BackupEnable();
}

XFUNC626( void,	S626_BackupEnableSet )( unsigned long hbd, 
                                       unsigned short en )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->BackupEnable( en );
}

XFUNC626( unsigned short,	S626_ChargeEnableGet )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->ChargeEnable();
}

XFUNC626( void,	S626_ChargeEnableSet )( unsigned long hbd, unsigned short en )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  return board->ChargeEnable( en );
}

// Watchdog functions:
XFUNC626( unsigned short,	S626_WatchdogTimeout )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->WatchdogTimeout();
}

XFUNC626( unsigned short,	S626_WatchdogEnableGet )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->WatchdogEnable();
}

XFUNC626( void,	S626_WatchdogEnableSet )( unsigned long hbd, unsigned short en )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  return board->WatchdogEnable( en );
}

XFUNC626( unsigned short,	S626_WatchdogPeriodGet )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0 );
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->WatchdogPeriod();
}

XFUNC626( void,	S626_WatchdogPeriodSet )( unsigned long hbd, unsigned short val )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WatchdogPeriod( val );
}

XFUNC626( void,	S626_WatchdogReset )( unsigned long hbd )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->WatchdogReset();
}

XFUNC626( void,	OpenIrqFifoBuffers )( unsigned long hbd, unsigned long irqId, \
                      unsigned long* regAddrs, long numAddrs, long* numBuffers )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  *numBuffers = board->OpenIrqFifoBuffers( irqId, regAddrs, numAddrs, *numBuffers );
}

XFUNC626( void,	CloseIrqFifoBuffers )( unsigned long hbd, unsigned long irqId )
{  
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return );
    board = boards[hbd];
    if( board.null() ) return;
  }
  board->CloseIrqFifoBuffers( irqId );
}

XFUNC626( unsigned long,	FetchIrqFifoBuffers )( unsigned long hbd, unsigned long irqId, \
                                    unsigned char* buffer, unsigned long bufferByteCnt )
{
  Cls626mm_Ptr board;
  {
    ACE_GUARD_REACTION( BOARD_MUTEX, mon, boardMutex, \
      rptBoardMutexFailure(__LINE__); return 0);
    board = boards[hbd];
    if( board.null() ) return 0;
  }
  return board->FetchIrqFifoBuffers( irqId, buffer, bufferByteCnt);
}

///////////////////////////////////////////////////////////////////////////////
// utility functions

XFUNC626( void, S626_DO_SetActive ) ( unsigned long hbd, unsigned short chan )
{
  WORD group = chan >> 4; // Group number associated with chan.
  WORD mask = 1 << ( chan & 15 ); // Bit mask for chan within its group.
  S626_DIOWriteBankSet( 0, group, S626_DIOWriteBankGet( 0, group ) | mask );
}

XFUNC626( void, S626_DO_SetInActive )( unsigned long hbd, unsigned short chan )
{
  WORD group = chan >> 4; // Group number associated with chan.
  WORD mask = 1 << ( chan & 15 ); // Bit mask for chan within its group.
  S626_DIOWriteBankSet( 0, group, S626_DIOWriteBankGet( 0, group ) & ~mask );
}

void rptBoardMutexFailure( int lineNum )
{
  ACE_DEBUG( (MY_DEBUG ACE_TEXT("Failed to acquire board list mutex; errno=%0X %m, \
                                line=%u\n"), ACE_OS::last_error(), lineNum) );
}